'use strict';

function findMinLexicographicalLCS(N1, N2, equals) {
    if (N1 <= 0 || N2 <= 0)
        return [];
    const dp = new Array(N1);
    for (let i = 0; i < N1; ++i)
        dp[i] = new Array(N2);
    {
        let first_i = 0;
        let first_j = 0;
        while (first_i < N1 && equals(first_i, 0) === false)
            ++first_i;
        while (first_j < N2 && equals(0, first_j) === false)
            ++first_j;
        dp[0].fill(0, 0, first_j);
        dp[0].fill(1, first_j, N2);
        for (let i = 0; i < first_i; ++i)
            dp[i][0] = 0;
        for (let i = first_i; i < N1; ++i)
            dp[i][0] = 1;
    }
    for (let i = 1; i < N1; ++i) {
        for (let j = 1; j < N2; ++j) {
            if (equals(i, j)) {
                dp[i][j] = dp[i - 1][j - 1] + 1;
            }
            else {
                const x = dp[i][j - 1];
                const y = dp[i - 1][j];
                dp[i][j] = x < y ? y : x;
            }
        }
    }
    const N = dp[N1 - 1][N2 - 1];
    const firsts = new Array(N + 1);
    for (let i = 0, j = N2 - 1, n = 1; n <= N; ++n) {
        while (dp[i][j] < n)
            ++i;
        firsts[n] = i;
    }
    const pairs = new Array(N);
    for (let j = N2 - 1, n = N; n > 0; --n) {
        let i = firsts[n];
        while (dp[i][j] < n)
            ++i;
        while (j >= 0 && dp[i][j] === n)
            --j;
        pairs[n - 1] = [i, j + 1];
    }
    return pairs;
}
function findLengthOfLCS(N1, N2, equals) {
    const dp = findLCSOfEveryRightPrefix(N1, N2, equals);
    return dp === null ? 0 : dp[N2 - 1];
}
function findLCSOfEveryRightPrefix(N1, N2, equals) {
    if (N1 <= 0 || N2 <= 0)
        return null;
    const dp = new Array(N2);
    {
        let first_j = 0;
        while (first_j < N2 && equals(0, first_j) === false)
            ++first_j;
        dp.fill(0, 0, first_j);
        dp.fill(1, first_j, N2);
    }
    for (let i = 1; i < N1; ++i) {
        let prev_ij = dp[0];
        if (dp[0] === 0 && equals(i, 0))
            dp[0] = 1;
        for (let j = 1; j < N2; ++j) {
            const z = dp[j];
            if (equals(i, j))
                dp[j] = prev_ij + 1;
            else {
                const y = dp[j - 1];
                dp[j] = y < z ? z : y;
            }
            prev_ij = z;
        }
    }
    return dp;
}

exports.findLCSOfEveryRightPrefix = findLCSOfEveryRightPrefix;
exports.findLengthOfLCS = findLengthOfLCS;
exports.findMinLexicographicalLCS = findMinLexicographicalLCS;
//# sourceMappingURL=index.cjs.map
